Module: Megatest::MultiProcess

Defined in:
lib/megatest/multi_process.rb

Overview

Fairly experimental multi-process queue implementation. It’s absolutely not resilient yet, if something goes a bit wrong in may fail in unexpected ways (e.g. hang or whatever). At this stage it’s only here to uncover what in the design need to be refactored to make multi-processing and test distribution work well (See the TODOs).

Defined Under Namespace

Classes: ClientQueue, Executor, InlineMonitor, Job, MessageSocket

Class Method Summary collapse

Class Method Details

.socketpairObject



17
18
19
# File 'lib/megatest/multi_process.rb', line 17

def socketpair
  UNIXSocket.socketpair(:SOCK_STREAM).map { |s| MessageSocket.new(s) }
end